class Theme:
    panel_title = "bright_blue"
    heading = "bold yellow"
    table_field = "bold bright_magenta"
    table_value = "none"
    inline_stat = "cyan"
    vendor_list = "cyan"
    nameserver_list = "cyan"
    disclaimer = "italic white on red"
    footer = "cyan"
    info = "bold green"
    warn = "bold yellow"
    error = "bold red"
